/*******************************************************************************
 *
 *  FILE NAME:  ITCCustomFilter.h
 *
 *  PURPOSE:    Defines the interface to custom .
 *
 *  COPYRIGHT (c) 2004-2015 HONEYWELL INTERNATIONAL INC., ALL RIGHTS RESERVED
 *
 ******************************************************************************/

#ifndef INCLUDED_ITCCUSTOMFILTER_H
#define INCLUDED_ITCCUSTOMFILTER_H

////////////////////////////////////////////////////////////////

// 
#define ITCFILTER_MORE_OUTPUT  (long)(0x01690001)
#define ITCFILTER_TOO_BIG      (long)(0xC104001B)

////////////////////////////////////////////////////////////////

#define ITCFILTER_CALLTYPE STDAPICALLTYPE

// Example: Declare a function with
//  ITCFILTER_API MyCompile( LPCWSTR szInput, DWORD nInputChars )
//  
#define ITCFILTER_API EXTERN_C __declspec(dllexport) long ITCFILTER_CALLTYPE

// Compile a filter.
// Return values:
//  ITCFILTER_SUCCESS indicates success.
//  Any negative code indicates error.
//
typedef long (ITCFILTER_CALLTYPE * ITCCompileFunc)(
	LPCWSTR szInput
    );

// Match data to a filter.
// Return values:
//  S_OK (zero) indicates success. szOutputBuffer contains the output.
//    If the function set *pnOutputChars to zero then there is no output.
//  ITCFILTER_MORE_OUTPUT indicates that szOutputBuffer contains the output.
//    Also causes this function to be called again with zero input to get
//    other output buffer.
//  ITCFILTER_TOO_BIG indicates that the output is larger than nOutputBufferChars.
//    The required output buffer characters is saved in *pnOutputChars.
//  Other negative code indicates error.
//   
//
typedef long (ITCFILTER_CALLTYPE * ITCFilterFunc)(
	LPCWSTR szInput,        // [in] input data
	DWORD   nInputChars,    // [in] number of input data characters
	LPWSTR  szOutputBuffer, // [out] output data
	PDWORD  pnOutputChars   // number of characters in [in] szOutputBuffer, [out] output data 
    );

#endif
////////////////////////////////////////////////////////////////
// END OF FILE:  ITCCustomFilter.h
//  COPYRIGHT (c) 2004 INTERMEC TECHNOLOGIES CORPORATION, ALL RIGHTS RESERVED
////////////////////////////////////////////////////////////////

