
/*******************************************************************************
 *
 *  FILE NAME:  ITCSSApi.h
 *
 *  PURPOSE:    Define the SmartSystems API for IDL.
 *
 *  AUTHOR:     Jeff Eaton
 *
 *  COPYRIGHT (c) 2005-2006 INTERMEC TECHNOLOGIES CORP., ALL RIGHTS RESERVED
 *
 ******************************************************************************/

#ifndef INCLUDED_ITCSSAPI_H
#define INCLUDED_ITCSSAPI_H

typedef long ITCSSAPI_RETURN_TYPE;

#if defined (_WIN32) || defined (WIN32)
	#define ITCSSAPI_CALL_TYPE STDAPICALLTYPE
	#define ITCSSAPI EXTERN_C
#else
	#error Unsupported platform
#endif

//3-31-2014
//Royal Mail customer has issue about their application not closing so
//engineer in UK found the method Uninit and suggested the customer
//to use the method however the this method was not mentioned in the
//DMRK User's Guide so now adding it to the User's Guide, but using
//the consistent name convention which is ITCSS+method name.
#define ITCSSUninit				Uninit

//Function definitions to match the character size for  
//the given mode the code it built in.
#if defined(UNICODE)

	#define ITCSSGet			ITCSSGetW
	#define ITCSSSet			ITCSSSetW
	#define ITCSSConfigFromFile	ITCSSConfigFromFileW
	
	#define ITCSSClearUserCounter		ITCSSClearUserCounterW
	#define ITCSSDeleteUserCounter		ITCSSDeleteUserCounterW
	#define ITCSSDecrementUserCounter	ITCSSDecrementUserCounterW
	#define ITCSSIncrementUserCounter	ITCSSIncrementUserCounterW
	#define ITCSSQueryUserCounter		ITCSSQueryUserCounterW

	#define ITCSSSendAssetMsg			ITCSSSendAssetMsgW
	
#else
	#define SZ_SS_SET			SZ_SS_SET_A

	#define ITCSSGet			ITCSSGetA
	#define ITCSSSet			ITCSSSetA
	#define ITCSSConfigFromFile	ITCSSConfigFromFileA

	#define ITCSSClearUserCounter		ITCSSClearUserCounterW
	#define ITCSSDeleteUserCounter		ITCSSDeleteUserCounterA
	#define ITCSSDecrementUserCounter	ITCSSDecrementUserCounterA
	#define ITCSSIncrementUserCounter	ITCSSIncrementUserCounterA
	#define ITCSSQueryUserCounter		ITCSSQueryUserCounterA

	#define ITCSSSendAssetMsg			ITCSSSendAssetMsgA
#endif // #if defined(UNICODE)


// *****************************************************************************************
// Description: 
//
//		ITCSSGet - Retrieves Intermec or customer SmartSystem information.
//
// Parameters:
//
//		pszGetData (in) - pointer to a null-terminated TCHAR string which
//			describe the data to retrieve.
//		
//		pszReturnedData (out) - pointer to a TCHAR string buffer used to store the
//			returned data.
//
//		pReturnedDataSize (in/out) - pointer to the number of characters that can be
//			stored in the pszReturnedData buffer.  If the data to be returned is 
//			larger than the pszReturnedData buffer, then pReturnedDataSize will be modified 
//			to contain the required size.
//
//		timeOut (in, optional) - time (0-4294967295 milliseconds) given to connect.
//			If zero is passed in, a default timeout value of 2000 ms will be used.
//			if INDEFINITE_TIMEOUT (defined as 4294967295) is used then the call will block
//			indefinitely until completion.
//
// Return:  Minimum set listed below.  Additional ones TBD.
//
//		E_SS_SUCCESS
//		E_SSAPI_FUNCTION_UNAVAILABLE
//		E_SSAPI_OPERATION_FAILED
//		E_SSAPI_MISSING_REQUIRED_PARM
//		E_SSAPI_MALFORMED_XML
//		E_SSAPI_TIMEOUT
//
// *****************************************************************************************
ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSGetA (const char *     pszGetData, 
		   char *           pszReturnedData, 
		   size_t *         pReturnedDataSize,
		   unsigned long    timeOut);

ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSGetW (const WCHAR *    pszGetData, 
		   WCHAR *        	pszReturnedData, 
		   size_t *       	pReturnedDataSize,
		   unsigned long  	timeOut);

// *****************************************************************************************
// Description: 
//
//		ITCSSSet - Sets Intermec or customer SmartSystem information.
//
// Parameters:
//
//		pszSetData (in) - pointer to a null-terminated TCHAR string which
//			describe the data to set.
//		
//		pszReturnedData (out) - pointer to a TCHAR string buffer used to store the
//			returned data.
//
//		pReturnedDataSize (in/out) - pointer to the number of characters that can be
//			stored in the pszReturnedData buffer.  If the data to be returned is 
//			larger than the pszReturnedData buffer, then pReturnedDataSize will be modified 
//			to contain the required size.
//
//		timeOut (in, optional) - time (0-4294967295 milliseconds) given to connect.
//			If zero is passed in, a default timeout value of 2000 ms will be used.
//			if INDEFINITE_TIMEOUT (defined as 4294967295) is used then the call will block
//			indefinitely until completion.
//
// Return:  Minimum set listed below.  Additional ones TBD.
//
//		E_SS_SUCCESS
//		E_SSAPI_FUNCTION_UNAVAILABLE
//		E_SSAPI_OPERATION_FAILED
//		E_SSAPI_MISSING_REQUIRED_PARM
//		E_SSAPI_MALFORMED_XML
//		E_SSAPI_TIMEOUT
//
// *****************************************************************************************
ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSSetA (const char * 	pszSetData, 
		   char *         	pszReturnedData, 
		   size_t *       	pReturnedDataSize,
		   unsigned long  	timeOut);

ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSSetW (const WCHAR *    pszSetData, 
		   WCHAR *        	pszReturnedData, 
		   size_t *       	pReturnedDataSize,
		   unsigned long  	timeOut);



// *****************************************************************************************
// Description: 
//
//		ITCSSConfigFromFile - Sets/Gets Intermec or customer SmartSystem information.
//
// Parameters:
//
//		pInCfgFileName (in) - pointer to a null-terminated TCHAR string which
//			contains the filename of the data to get/set.
//		
//		pOutCfgFileName (in) - pointer to a null-terminated TCHAR string which
//			contains the filename of the retrieved data or the results of the set.
//		
//		pszReturnedData (out) - pointer to a TCHAR string buffer used to store the
//			returned data.
//
//		pReturnedDataSize (in/out) - pointer to the number of characters that can be
//			stored in the pszReturnedData buffer.  If the data to be returned is 
//			larger than the pszReturnedData buffer, then pReturnedDataSize will be modified 
//			to contain the required size.
//
//		timeOut (in, optional) - time (0-4294967295 milliseconds) given to connect.
//			If zero is passed in, a default timeout value of 2000 ms will be used.
//			if INDEFINITE_TIMEOUT (defined as 4294967295) is used then the call will block
//			indefinitely until completion.
//
// Return:  Minimum set listed below.  Additional ones TBD.
//
//		E_SS_SUCCESS
//		E_SSAPI_FUNCTION_UNAVAILABLE
//		E_SSAPI_OPERATION_FAILED
//		E_SSAPI_MISSING_REQUIRED_PARM
//		E_SSAPI_MALFORMED_XML
//		E_SSAPI_TIMEOUT
//
// *****************************************************************************************
ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSConfigFromFileA (const char * 		pInCfgFileName, 
					  const char * 		pOutCfgFileName,
					  char * 			pszReturnedData,
					  size_t * 			pReturnedDataSize,
					  unsigned long   	timeOut);

ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSConfigFromFileW (const WCHAR * 	pInCfgFileName, 
					  const WCHAR * 	pOutCfgFileName,
					  WCHAR * 			pszReturnedData,
					  size_t * 			pReturnedDataSize,
					  unsigned long   	timeOut);


// *****************************************************************************************
// Description: 
//
//		ITCSSSendAssetMsg - Trigger to send a Smart Systems Asset message.
//
// Parameters:
//
//		pszReason(in) - pointer to a null-terminated TCHAR string which specifies why the
//			Asset Message is being sent.
//
// Return:  Minimum set listed below.  Additional ones TBD.
//
//		E_SS_SUCCESS
//		E_SSAPI_FUNCTION_UNAVAILABLE
//		E_SSAPI_OPERATION_FAILED
//
// *****************************************************************************************
ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSSendAssetMsgA (const char * pszReason);

ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSSendAssetMsgW (const WCHAR * pszReason);

// *****************************************************************************************
// Description: 
//
//		ITCSSDeleteUserCounter - Deletes a user counter.  The counter will no longer be 
//			included in the Asset message.
//
// Parameters:
//
//		pszName(in) - pointer to a null-terminated TCHAR string which specifies the 
//			name of the counter.
//
// Return:  Minimum set listed below.  Additional ones TBD.
//
//		E_SS_SUCCESS
//		E_SSAPI_FUNCTION_UNAVAILABLE
//		E_SSAPI_OPERATION_FAILED
//
// *****************************************************************************************
ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSDeleteUserCounterA (const char * pszName);

ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSDeleteUserCounterW (const WCHAR * pszName);

// *****************************************************************************************
// Description: 
//
//		ITCSSClearUserCounter - Clears a user counter. The counter will be set to zero,
//
// Parameters:
//
//		pszName(in) - pointer to a null-terminated TCHAR string which specifies the 
//			name of the counter.
//
// Return:  Minimum set listed below.  Additional ones TBD.
//
//		E_SS_SUCCESS
//		E_SSAPI_FUNCTION_UNAVAILABLE
//		E_SSAPI_OPERATION_FAILED
//
// *****************************************************************************************
ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSClearUserCounterA (const char * pszName);

ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSClearUserCounterW (const WCHAR * pszName);

// *****************************************************************************************
// Description: 
//
//		ITCSSDecrementUserCounter - Decrements a user counter.  When the counter reaches zero,
//			it will stay at that value until incremented.  A user counter will not go negative.
//
// Parameters:
//
//		pszName(in) - pointer to a null-terminated TCHAR string which specifies the 
//			name of the counter.
//
// Return:  Minimum set listed below.  Additional ones TBD.
//
//		E_SS_SUCCESS
//		E_SSAPI_FUNCTION_UNAVAILABLE
//		E_SSAPI_OPERATION_FAILED
//
// *****************************************************************************************
ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSDecrementUserCounterA (const char * pszName);

ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSDecrementUserCounterW (const WCHAR * pszName);

// *****************************************************************************************
// Description: 
//
//		ITCSSIncrementUserCounter - Increments a user counter.  When the counter reaches the
//			maximum value, it will roll over to 1. The value of a counter can be from 0 to
//			4294967295. (the size of an unsigned long)
//
// Parameters:
//
//		pszName(in) - pointer to a null-terminated TCHAR string which specifies the 
//			name of the counter.
//
// Return:  Minimum set listed below.  Additional ones TBD.
//
//		E_SS_SUCCESS
//		E_SSAPI_FUNCTION_UNAVAILABLE
//		E_SSAPI_OPERATION_FAILED
//
// *****************************************************************************************
ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSIncrementUserCounterA (const char * pszName);

ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSIncrementUserCounterW (const WCHAR * pszName);

// *****************************************************************************************
// Description: 
//
//		ITCSSQueryUserCounter - Query the value of a user counter.  
//
// Parameters:
//
//		pszName(in) - pointer to a null-terminated TCHAR string which specifies the 
//			name of the counter.
//
//		pValue (out) - pointer to the value of the counter.
// Return:  Minimum set listed below.  Additional ones TBD.
//
//		E_SS_SUCCESS
//		E_SSAPI_FUNCTION_UNAVAILABLE
//		E_SSAPI_OPERATION_FAILED
//
// *****************************************************************************************
ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSQueryUserCounterA (const char * pszName, unsigned long *pValue);

ITCSSAPI 
ITCSSAPI_RETURN_TYPE ITCSSAPI_CALL_TYPE 
ITCSSQueryUserCounterW (const WCHAR * pszName, unsigned long *pValue);

ITCSSAPI 
void ITCSSAPI_CALL_TYPE 
Uninit ();

#endif // INCLUDED_ITCSSAPI_H
//
//  END OF FILE:  ITCSSAPI.h
//  COPYRIGHT (c) 2005-2006 INTERMEC TECHNOLOGIES CORP., ALL RIGHTS RESERVED
//  
