/*******************************************************************************
 *
 *  FILE NAME:  Itc50.h
 *  PURPOSE:    Provides public interfaces of the Mobility SDK. 
 *
 *  Copyright (c) 1998-2015 Honeywell International Inc. All Rights Reserved.
 *
 ******************************************************************************/

#ifndef INCLUDED_ITC50_H                /* Has this file been included before?  */
#define INCLUDED_ITC50_H                /* No, remember it has been now         */


////////////////////////////////////////////////////////////////
//  #INCLUDES   
//
#include "itc50MsgSys.h"
//#include "MessageResourceMsgSys.h"
//#include <tchar.h>
//#include <objbase.h>
#include <windows.h>

//#ifdef __cplusplus
//x#ifdef UNDER_CE
//x#include <WceAtl.h>
//x#endif 
//x#endif

#include <winsock.h>
//x#include <af_irda.h>
#ifdef _UDPP_
#include "ITCSwinsock.h"
#endif
//rl 3-30-2005 substituted IADCDevice.h for "IBarCodeReader.h" because the later one causes issues if included in multiple files
//#include "IBarCodeReader.h"
#include "IADCDevice.h"

//
////////////////////////////////////////////////////////////////
//  #DEFINES   
//      QuickWin
//

#ifndef __cplusplus
#define bool unsigned char
#define true ( (unsigned char)1)
#define false ( (unsigned char)0)
#endif

// return values for ITCGetDeviceType()
#define ITC_DEVICE_UNKN 0
#define ITC_DEVICE_CK30 1
#define ITC_DEVICE_CK31 2
#define ITC_DEVICE_751G 3
#define ITC_DEVICE_CN2G 4
#define ITC_DEVICE_700_COLOR 5
#define ITC_DEVICE_700COLOR_PPC2003 6
#define ITC_DEVICE_XP_CV60 7
#define ITC_DEVICE_CV60_CE 8
#define ITC_DEVICE_CK60 9
#define ITC_DEVICE_CN30 10
#define ITC_DEVICE_CN2B 11
#define ITC_DEVICE_CN3  12
#define ITC_DEVICE_CV30 13
#define ITC_DEVICE_CK32	14
#define ITC_DEVICE_CK3  15
#define ITC_DEVICE_CN4  16
#define ITC_DEVICE_CN50 17
#define ITC_DEVICE_CS40	18

#define ITC_DEVICE_CN70  20  //Left small number space for any more Andromeda II devices for simplicity of grouping
#define ITC_DEVICE_CN70e 21  
#define ITC_DEVICE_CK70  22
#define ITC_DEVICE_CK71  23
#define ITC_DEVICE_CN51  24
#define ITC_DEVICE_CV41  25
#define ITC_DEVICE_CV61  26
#define ITC_DEVICE_CV31  27
#define ITC_DEVICE_VM3   28



//following are generic and up in the 162 decimal range
#define ITC_DEVICE_AII  0xA2
#define ITC_DEVICE_AIII  0xA3
 


// Fixes bug CEDev00003580.
// ITC_TO_MANY_DEVICES should be pulled in by itc50MsgSys.h
#define ITC_TOO_MANY_DEVICES ITC_TO_MANY_DEVICES

#define ITC_ERASE_FIELD       1  
#define ITC_RETURN_ON_TAB     2  
#define ITC_RETURN_ON_FULL    4  
#define ITC_RETURN_ON_FUNCTION   8  
#define  ITC_DISPLAY_ONLY     16 
#define  ITC_AT_END           32
#define ITC_NO_DISPLAY        64
#define ITC_RETURN_ON_ESC     128   
#define  ITC_UPCASE           256   
#define  ITC_LOCASE           512   
#define  ITC_RETURN_ON_UD_ARROWS 1024
#define  ITC_STAY_IN_OVERSTRIKE  2048
#define  ITC_STAY_IN_INSERT      4096
#define  ITC_RETURN_ON_COM    8192
#define  ITC_RETURN_ON_ENTER     16384

#define MAKE_QW_LEN(buflen, winsize)      ((UINT)(((WORD)(buflen)) | ((UINT)((WORD)(winsize))) << 16))

#define  ITC_NORMAL     0
#define  ITC_BOLD    1
#define  ITC_ITALIC     2
#define  ITC_UNDERLINE  4
#define  ITC_INVERSE    8


#define WM_ITC_RECEIVED_DATA     WM_USER + 20

#define ITC_FROM_LABEL           0  /* Input device selected */
#define ITC_FROM_KEYBOARD        1  /* Keyboard selected  */
#define ITC_FROM_SERIAL_SELECT      2  /* Serial Port selected */
#define ITC_FROM_IRDA_SELECT     3  /* Infrared port selected */
#define ITC_FROM_TCPIP_SELECT    4  /* TCP/IP network */
#define ITC_FROM_UDPPLUS_SELECT     5  /* UDP Plus network */
#define ITC_FROM_SMARTCARD_SELECT   6  /* Smart Card */
#define ITC_FROM_RFID_SELECT     7  /* RFID */

#define ITC_MAX_INPUT_DEVICES    256

#define ITC_NO_SELECT            0x00  /* No Select */
#define ITC_LABEL_SELECT         0x01 // TEXT("Label") //0x01  /* Input device selected */
#define ITC_KEYBOARD_SELECT         0x02  /* Keyboard selected  */
#define ITC_SERIAL_SELECT        0x04  /* Serial Port selected */
#define ITC_IRDA_SELECT          0x08  /* Infrared port selected */
#define ITC_TCPIP_SELECT         0x10  /* TCP/IP network */
#define ITC_UDPPLUS_SELECT       0x20  /* UDP Plus network */
#define ITC_SMARTCARD_SELECT     0x40  /* Smart Card */
#define ITC_RFID_SELECT          0x80  /* RFID */
#define ITC_INPUT_DEVICE_SELECT     ITC_LABEL_SELECT |       \
                           ITC_SMARTCARD_SELECT |   \
                           ITC_RFID_SELECT
/* Pre/Post amble maximum length */
#define AMBLE_MAX_LENGTH                20

/*
 * These #defines correspond to the dwMethod for ITCSetEOM()
 */
#define ITC_NO_EOM               0  /* No end of message */
#define ITC_ONE_EOM              1  /* One end of message */
#define ITC_TWO_EOM              2  /* Two end of message */
#define ITC_TCP_LENGTH           3  /* Message length in first two bytes */
#define ITC_TCP_LEN_NETORDER     4  /* Message length in first two bytes, Two bytes will be swapped. */

#define ITC_ADC_SUB_KEY_STRING      TEXT("Merlin1")
/*
 * These #defines related to timeout for receive and transmit.
 */
#define MAX_IRDA_DEVICE_ID       3
#define MAX_RETRY                5
#define MAX_DEVICES              20
#define SZ_DEVICES_NAME          80
#define SZ_MAX_BUFFER            2048
#define SZ_MAX_STRING            256

////////////////////////////////////////////////////////////////
//   ENUMS   
//
////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////
//   CLASSES/STRUCTS
//      NONE
//
////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////
//   FUNCTIONS
//
////////////////////////////////////////////////////////////////

#ifdef __cplusplus
class ITCTftpClient;
                                    extern "C"{
#endif 



INT ITCGetDeviceType();

INT ITCGetDeviceTypeGen();  //returns ITC_DEVICE_AII for CN3 and above to simplify device specific functions to a family



//------------------------------------------------------------------------------------------------------------------
// Message functions
//------------------------------------------------------------------------------------------------------------------

VOID ITCMessage(HRESULT hStatusCode, va_list * ArgumentList, bool bArray);
HRESULT ITCMessageWithoutDisplay(HRESULT hStatusCode, va_list * ArgumentList, 
                                bool bArray, LPTSTR szMessageString);



//------------------------------------------------------------------------------------------------------------------
// Virtual Wedge functions
//------------------------------------------------------------------------------------------------------------------

HRESULT ITCWedgeEnable (LPCTSTR  lpDevice);
HRESULT ITCWedgeDisable (LPCTSTR lpDevice);
HRESULT ITCWedgeIsEnabled (LPCTSTR lpDevice, bool* pbIsEnabled);
HRESULT ITCWedgeIsDisabled (LPCTSTR lpDevice, bool* pbIsDisabled);
HRESULT ITCWedgeSetDataGrid (LPCTSTR lpDevice, LPCTSTR lpDataGrid);
HRESULT ITCWedgeGetDataGrid (LPCTSTR lpDevice, LPTSTR lpDataGrid);
HRESULT ITCWedgeSetPreamble (LPCTSTR lpDevice, LPCTSTR lpPreambleString);
HRESULT ITCWedgeGetPreamble (LPCTSTR lpDevice, LPTSTR lpPreambleString);
HRESULT ITCWedgeSetPostamble (LPCTSTR lpDevice, LPCTSTR lpPostambleString);
HRESULT ITCWedgeGetPostamble (LPCTSTR lpDevice, LPTSTR lpPostambleString);
HRESULT ITCWedgeSetCodePage (LPCTSTR lpDevice, DWORD dwCodePage);
HRESULT ITCWedgeGetCodePage (LPCTSTR lpDevice, LPDWORD lpnCodepage);

//--------------------------------------------------------------------------------------------
// General Scanner functions
//--------------------------------------------------------------------------------------------
typedef enum scannerDevice {
    DEFAULT_INTERNAL = 0,
    DEFAULT_EXTERNAL,
    DOCK_TETHERED
} SCANNER_DEVICE;

HRESULT ITCSetScannerPreamble  (SCANNER_DEVICE eDevice, LPCTSTR lpPreambleString);
HRESULT ITCGetScannerPreamble  (SCANNER_DEVICE eDevice, LPTSTR lpPreambleString);
HRESULT ITCSetScannerPostamble (SCANNER_DEVICE eDevice, LPCTSTR lpPostambleString);
HRESULT ITCGetScannerPostamble (SCANNER_DEVICE eDevice, LPTSTR lpPostambleString);

//------------------------------------------------------------------------------------------------------------------
// quickwin
//------------------------------------------------------------------------------------------------------------------


HWND ITCCreateScreen(HINSTANCE hInstance);
HWND ITCCreateScreenEx(HINSTANCE hInstance, TCHAR *szUserFont, int Fontheight);
HRESULT ITCSetCursorXY (HWND whnd,int X,int Y);
HRESULT ITCGetCursorXY (HWND whnd,int *X,int *Y);
HRESULT   ITCReceiveField( HWND  whnd,
                  TCHAR **Connections,
                  ULONG uTimeout,
                  DWORD dwAttributes,
                  DWORD dwFlags,
                  UINT  uLength,
                  LPTSTR szReturnedConnection,
                  LPTSTR lpszDataBuffer);

HRESULT   ITCReceiveFieldEx(  HWND  whnd,
                  DWORD ConnID,
                  ULONG uTimeout,
                  DWORD dwAttributes,
                  DWORD dwFlags,
                  UINT  uLength,
                  LPDWORD dwReturnedConnID,
                  LPTSTR lpszDataBuffer);


HRESULT ITCPuts(HWND whnd,LPTSTR Text, DWORD dwAttributes);
HRESULT ITCButton(HWND whnd,LPTSTR Text,DWORD dwFlags);
HRESULT ITCListBox(HWND whnd,TCHAR *szItems[], DWORD dwFlags,int Height, int *Selection);
void ITCClearScreen (HWND whnd);
HRESULT ITCDestroyScreen (HWND whnd);
HRESULT ITCGetch (HWND whnd, DWORD *Keycode, ULONG Timeout);




//------------------------------------------------------------------------------------------------------------------
// system information
//------------------------------------------------------------------------------------------------------------------

HRESULT ITCPowerStatus (LPDWORD lpdwLineStatus, LPDWORD lpdwBatteryStatus,
                  LPDWORD lpdwBackupStatus, UINT   *puFuelGauge);


//------------------------------------------------------------------------------------------------------------------
// Reader command functions
//------------------------------------------------------------------------------------------------------------------

HRESULT ITCCommand(LPCTSTR lpszCommand);
HRESULT ITCGetConfigInfo( LPTSTR lpszConfigString, LPDWORD uLength);

//------------------------------------------------------------------------------------------------------------------
// ITCUtility functions
//------------------------------------------------------------------------------------------------------------------

typedef BOOL (CALLBACK* SUSPENDPROC)(void);

BOOL ITCInitSuspend(DWORD Seconds, SUSPENDPROC);
BOOL ITCResetSuspend(void);
BOOL ITCCleanupSuspend(void); 

HRESULT ITCSuspend(void);
HRESULT ITCWarmBoot(void);
HRESULT ITCColdBoot(void);

HRESULT  ITCDisplayHeaterEnableStatus (bool *pEnabled);
HRESULT ITCControlDisplayHeater (BOOL on);

HRESULT ITCCreateProcessAndWait(LPCTSTR Directory, LPCTSTR CmdLine, LPCTSTR Parameters, long Timeout);
HRESULT ITCCreateProcessAndReturn(LPCTSTR Directory, LPCTSTR CmdLine, LPCTSTR Parameters);

HRESULT ITCGetSerialNumber(LPTSTR number, int buffSize);


/************************************************************************************************************************/

/************************************************************************************************************************/

HRESULT ITCSwitchSerialDock(char Dock);
HRESULT ITCDisableRegFlushKey(void);
HRESULT ITCEnableRegFlushKey(void);
HRESULT ITCSetRegFlushKeyLocation(int Location);
HRESULT ITCGetRegFlushKeyLocation(int *Location);
HRESULT ITCGetStatusRegFlushKey(BOOL *Enable);
HRESULT ITCRegFlushKey(void);
HRESULT ITCGetRadioStatus(BOOL *Online);
HRESULT ITCGetSignalStrength(ULONG *Signal);
HRESULT ITCGetLinkSpeed(int *Speed);


/************************************************************************************************************************/

/************************************************************************************************************************/


//------------------------------------------------------------------------------------------------------------------
// ITCLight functions
//------------------------------------------------------------------------------------------------------------------
//x #ifdef INTERMEC70_MONO
//
// --- Keypad Light Functions ---
//
void ITCSetKeyboardLight(BOOL bKeyLight);

//
// --- ScreenLight Functions ---
//
//
// PUBLIC API defines (do not alter)
//
#define POWER_MODE_BATTERY      0xDC
#define POWER_MODE_DC           0xDC
#define POWER_MODE_AC           0xAC
#define SCREENLIGHT_ON                  TRUE
#define SCREENLIGHT_OFF                 FALSE
#define ENABLE_SCRNLGHT_OPTION          TRUE
#define DISABLE_SCRNLGHT_OPTION         FALSE
#define CURRENT_BRIGHTNESS_SETTING      TRUE
#define FORCE_NEW_BRIGHTNESS_SETTING    FALSE

HRESULT 
ITCSetScreenLight(
    BOOL fON_OFF,               // IN: SCREENLIGHT_ON | SCREENLIGHT_OFF
    BOOL fUseCurrentBrightness  // IN: CURRENT_BRIGHTNESS_SETTING | FORCE_NEW_BRIGHTNESS_SETTING
        // Only relevant for fON_OFF == SCREENLIGHT_ON...
        // TRUE  = Assume the current brightness level 
        // FALSE = Set a new brightness level based on latest call to 
        //         SetScreenBrightness().
    );

HRESULT 
ITCGetScreenLight(
    BOOL* pfON_OFF          // OUT: Same as above.
    );

HRESULT 
ITCSetScreenLightTimeout(
    DWORD dwPowerSelect,    // IN: POWER_MODE_DC | POWER_MODE_AC
    DWORD dwTimeout         // IN: seconds, 0 to -1, zero is infinity.
    );

HRESULT 
ITCGetScreenLightTimeout(
    DWORD  dwPowerSelect,   // IN: POWER_MODE_DC | POWER_MODE_AC
    DWORD* pdwTimeout       // OUT: seconds, 0 to -1, zero is infinity.
    );

HRESULT 
ITCSetScreenLightEnableOnUserAction(
    DWORD dwPowerSelect,// IN: POWER_MODE_DC | POWER_MODE_AC
    BOOL  dwEnable      // IN: ENABLE_SCRNLGHT_OPTION or DISABLE_SCRNLGHT_OPTION
    );

HRESULT 
ITCGetScreenLightEnableOnUserAction(
    DWORD dwPowerSelect,    // IN: POWER_MODE_DC | POWER_MODE_AC
    DWORD* pdwEnable        // OUT
    );

HRESULT 
ITCGetMaxScreenBrightness(
    DWORD* pdwBrightness    // OUT: Returns max-supported brightness index.
    );

HRESULT 
ITCSetScreenBrightness(
    DWORD dwBrightness      // IN: Range = { 1 - ITCGetMaxScreenBrightness() }
    );

HRESULT 
ITCGetScreenBrightness(
    DWORD* pdwBrightness    // OUT: Same as above.
    );

HRESULT 
ITCSetScreenBrightnessAcDc(
    DWORD dwPowerSelect,    // IN: POWER_MODE_DC | POWER_MODE_AC
    DWORD dwBrightness      // IN: Range = { 1 - ITCGetMaxScreenBrightness() }
    );

HRESULT 
ITCGetScreenBrightnessAcDc(
    DWORD dwPowerSelect,    // IN: POWER_MODE_DC | POWER_MODE_AC
    DWORD* pdwBrightness    // OUT: Same as above.
    );

HRESULT 
ITCSetScreenLightEnableRestoreOnPowerUp(
    BOOL  fEnable   // IN: ENABLE_SCRNLGHT_OPTION | DISABLE_SCRNLGHT_OPTION
                    //     Restore screenLight-state when device is turned on.
    );

HRESULT 
ITCGetScreenLightEnableRestoreOnPowerUp(
    BOOL* pfEnable  // OUT: Same as above.
    );

// NOTE: The following feature is not yet implemented pending availability.
//
HRESULT 
ITCSetScreenLightEnableAmbientSensor(
    BOOL  fEnable   // IN: ENABLE_SCRNLGHT_OPTION | DISABLE_SCRNLGHT_OPTION
                    // ENABLE = ScreenLight auto-on when ambient light fades.
    );

HRESULT 
ITCGetScreenLightEnableAmbientSensor(
    BOOL* pfEnable  // OUT
    );

// --- END ScreenLight functions ---
//
//x # endif // INTERMEC700_COLOR
//x #endif // INTERMEC700_MONO




//------------------------------------------------------------------------------------------------------------------
// ITCMixer - Audio Mixer functions
//
// Note: ITCMixer functions support the 700-WM2003 and 700-WM5.0 product families only.
//       ITCMixer functions have been discontinued for all other device types as of 
//       April 2008 (deprecation is retro-active for all other device types).
//
// For all Intermec computing devices other than 7xx-WM2003 and 7xx-WM5.0:
//
//       If you need to control speaker/headphone gain, see the Device Resource Kit's 
//       User's Guide section entitled "Audio And Sound Functions" for further details.
//
//       If you need to control microphone gain, recording gain, or any audio feature other 
//       than speaker / headphone gain, please contact Honeywell Developer Support and ask 
//       for the appropriate platform-level API's for your particular device type. 
//
//------------------------------------------------------------------------------------------------------------------

// TYPEDEFS
//
typedef long  CHANNEL_LIST;
typedef long  CHANNEL_ID;

typedef enum _PRESET_ID {
    SID_NOT_DEFINED = 0,
    SID_DEFAULT_PRESET = 0,
    SID_USER = 1,
    SID_USER_LIMIT = 0x7fff,
    SID_RESERVED = 0x8000,
    SID_RESERVED_LIMIT = 0xffff,
    SID_MAX = 0x10000,
    SID_ERROR   = -1
} PRESET_ID, *P_PRESET_ID ;


// Audio Mixer Channel ID's
//

// Input Channels used to set or get mixer input gain and mute.
//
#define ITC_SDK_MIXER_IN_MICROPHONE     0x1
#define ITC_SDK_MIXER_IN_WAN_RADIO      0x2
#define ITC_SDK_MIXER_IN_STEREO_PCM     0x4
#define ITC_SDK_MIXER_IN_SCANNER_BEEP   0x8

// Output Channels used to set or get mixer output volume and mute.
//
#define ITC_SDK_MIXER_OUT_SPEAKER       0x10
#define ITC_SDK_MIXER_OUT_HEADPHONE     0x20
#define ITC_SDK_MIXER_OUT_WAN_RADIO     0x40
#define ITC_SDK_MIXER_OUT_RECORD_SOURCE 0x80
#define ITC_SDK_MIXER_MASTER_VOLUME     0x1000

// Recording sources = { Microphone | Radio }
//
#define ITC_SDK_MIXER_SOURCE_MICROPHONE            0x100
#define ITC_SDK_MIXER_SOURCE_WAN_RADIO             0x200
#define ITC_SDK_MIXER_SOURCE_LINEAR_GAIN_MONO_MIX  0x400

// API Functions
//
HRESULT 
ITCMixerInit( void );

HRESULT 
ITCMixerSetMicBoost( BOOL a_fEnable );

HRESULT 
ITCMixerGetMicBoost( BOOL* a_pfEnable );

HRESULT 
ITCMixerSetInputGain( 
    CHANNEL_LIST* pidMultChannel, // IN/OUT: OR'ed bit-list of input channels to update.
                            //  Any channels failing the update are output to this parm.
    long  lGain             // IN: Percent gain to be applied to each channel in the list.
    );

HRESULT 
ITCMixerGetInputGain( CHANNEL_ID idSingleChannel, long* plGain );

HRESULT 
ITCMixerSetOutputGain( 
    CHANNEL_LIST* pidMultChannel, // IN/OUT: OR'ed bit-list of output channels to update.
                            //  Any channels failing the update are output to this parm.
    long  lGain             // IN: Percent gain to be applied to each channel in the list.
    );

HRESULT 
ITCMixerGetOutputGain( 
    CHANNEL_ID idSingleChannel, 
    long* plGain 
    );

HRESULT 
ITCMixerSetMute( 
        // IN/OUT: OR'ed bit-list of channels to mute and/or unmute.
        //  Any channels failing the update are output to this parm.
    CHANNEL_LIST* pidMultChannels 
    );

HRESULT 
ITCMixerGetMute( CHANNEL_LIST* pidMultChannels );

HRESULT 
ITCMixerSelectRecordSource( CHANNEL_ID idSingleChannel );

HRESULT 
ITCMixerGetRecordSource( CHANNEL_ID* pidSingleChannel );

HRESULT 
ITCMixerApplyPreset( PRESET_ID a_idCfg );

HRESULT 
ITCMixerSavePreset( 
    PRESET_ID a_idCfg ,         // IN: Caller's configuration ID.
    const TCHAR* pwszCfgName    // IN: Optional, Ptr to abitrary string (80 chars max)
    );

HRESULT 
ITCMixerGetPresetName( 
    PRESET_ID a_idCfg ,         // IN: Caller's configuration ID.
    TCHAR* pwszCfgName          // OUT:Ptr to caller's string buffer.
    );

// --- END Audio Mixer functions ---
//


//------------------------------------------------------------------------------------------------------------------
// Communication functions
//------------------------------------------------------------------------------------------------------------------
/*
 * This defines the structure that is returned from ITCOpenConnection()
 */
typedef struct _OutputBuffer
{
   DWORD    dwOrigin;               // Connection Type
   TCHAR    szOrigin[SZ_DEVICES_NAME]; // full description of connection..
    TCHAR      szDataBuffer[SZ_MAX_BUFFER];// Data buffer passed back from work thread
    ULONG      nDataLen;               // Data Buffer size 
   int         nMethod;             // Type of EOM
   char     cEOM1;                  // First optional EOM character
   char     cEOM2;                  // Second optional EOM character
   int         EOMs;                // Number of unprocessed EOMs
    char    TmpBuffer[SZ_MAX_BUFFER];  // Data buffer passed back from work thread
   int         TmpOffset;              // Index to first valid char in temp buffer
   int         TmpEnd;                 // Index to last valid char in temp buffer

       
    ITC_BARCODE_SYMBOLOGY_ID eSymbology;  // Bar Code Symbology ID
    ITC_BARCODE_DATATYPE eDataType;       // Bar Code Data Type
   DWORD    ConnID;                 // connection handle which returned the data
}tOutputBuffer, *tpOutputBuffer;
 
HRESULT 
ITCReceiveInput ( HWND          hWnd, 
              int         TotalDevices, 
              TCHAR          *szDevicesNames[], 
              ULONG          uTimeOut, 
              tpOutputBuffer  pOutputBuffer );

HRESULT 
ITCReceiveInputEx( HWND         hWnd, 
              DWORD        ConnIDs,
              ULONG          uTimeOut, 
              tpOutputBuffer  pOutputBuffer );

HRESULT 
ITCOpenConnection ( DWORD dwOrigin,
                    LPCTSTR lpszAddress,
                    UINT uPortID,
                    LPCTSTR irdaServerName);

HRESULT 
ITCOpenConnectionEx ( DWORD dwOrigin,
                    LPCTSTR lpszAddress,
                    UINT uPortID,
                    LPCTSTR irdaServerName,
               DWORD *lpdeHandle);

HRESULT 
ITCTransmitBuffer( LPCTSTR lpszUserAddress, 
                LPCTSTR lpszDataBuffer, 
               UINT uLength,
               ULONG uTimeout );
HRESULT 
ITCTransmitBufferEx(DWORD    ConnID, 
               LPCTSTR lpszDataBuffer, 
               UINT uLength,
               ULONG uTimeout );

HRESULT 
ITCSetEom(int  nMethod, 
        char   cEOM1, 
        char   cEOM2 );

HRESULT 
ITCSetEOMEx(int      nMethod, 
         char  cEOM1, 
         char  cEOM2,
         DWORD ConnID);

void  ITCCloseAllConnections();

void ITCCloseConnectionsEx(DWORD Handles);

void ITCCloseAllConnectionsEx();

#ifdef __cplusplus
}
#endif

////////////////////////////////////////////////////////////////////////////////
// TFTP declarations
////////////////////////////////////////////////////////////////////////////////
typedef 
enum tagTftpMode
    { ITC_TFTPMODE_UNKNOWN = 0,
   ITC_TFTPMODE_OCTET   = 1,
   ITC_TFTPMODE_NETASCII   = 2
    } ITC_TFTPMODE;

typedef 
enum tagTftpState
    { ITC_TFTPSTATE_NOT_BEGUN = 0,
   ITC_TFTPSTATE_IN_PROGRESS  = 1,
   ITC_TFTPSTATE_DONE_OK   = 2,
   ITC_TFTPSTATE_DONE_SVR_ERROR  = 3,
   ITC_TFTPSTATE_DONE_CLT_ERROR  = 4,
   ITC_TFTPSTATE_DONE_CANCEL  = 5
    } ITC_TFTPSTATE;


typedef struct  tagItcTftpStatus
    {
    WORD wSize;
    WORD wBlocksOk;
    WORD wBlocksFailed;
    ITC_TFTPSTATE eState;
    WORD wErrorCode;
    TCHAR szErrorMsg[ 100 ];
    } ITC_TFTPSTATUS;


typedef 
enum tagTftpProp
    { ITC_TFTPPROP_UNKNOWN = 0,
   ITC_TFTPPROP_TIMEOUT = 1,
   ITC_TFTPPROP_RESENDLIMIT   = 2,
   ITC_TFTPPROP_BLKSIZE = 3,
   ITC_TFTPPROP_XOPTIONS   = 4
    } ITC_TFTPPROP;


typedef struct {                        //TFTP request
    WORD            wOpCode;            //  Op code TFTP_OP_RRQ or TFTP_OP_WRQ
    char            szFileName[1];      //  1st char of filename
                                        //  Mode follows filename
                                        //  Option Extensions follow mode
    }   TFTP_REQUEST;

typedef struct {                        //TFTP data
    WORD            wOpCode;            //  Op code TFTP_OP_DATA
    WORD            wBlockNum;          //  Block number
    BYTE            szData[1];          //  1st char of data
    }   TFTP_DATA;

typedef struct {                        //TFTP acknowledgement
    WORD            wOpCode;            //  Op code TFTP_OP_ACK
    WORD            wBlockNum;          //  Block number
    }   TFTP_ACK; 

typedef struct {                        //TFTP acknowledgement
    WORD            wOpCode;            //  Op code TFTP_OP_ERROR
    WORD            wErrorCode;         //  Error code
    char            szMsg[1];           //  1st character of error message
    }   TFTP_ERROR; 

typedef struct {                        //TFTP Option Acknowledgement (RFC 2347)
    WORD            wOpCode;            //  Op code TFTP_OP_OACK
    char            szOption[1];        //  1st char of 1st option name
    }   TFTP_OACK; 

typedef struct {                        //Map TFTP messages by OP code
      WORD              wLenMax;        //  Alloced length of block in u (below)
      WORD              wLenData;       //  Length of actual data in block u
      union {
        WORD            wOpCode;        //  Op code TFTP_OP_...
        char            cMsg;           //  TFTP message start
        TFTP_REQUEST    request;        //  Request 
        TFTP_DATA       data;           //  Data 
        TFTP_ACK        ack;            //  Acknowledgement
        TFTP_ERROR      error;          //  Error
        TFTP_OACK       oack;           //  Option Acknowledgement
      } u;
    }   TFTP_BLOCK;

typedef struct {
    ULONG RFCExt;
    ULONG ResendLimit;
    ULONG BlkSize;  
    ULONG Timeout;
} TFTP_EX_PARMS;

#define TFTP_RFC_DEFAULT 0;
#define TFTP_RESEND_DEFAULT 100;
#define TFTP_BLKSIZE_DEFAULT 2840; // 2*TFTP_BLOCK_SIZE_MTU
#define TFTP_TIMEOUT_DEFAULT 500;

////////////////////////////////////////////////////////////////
//   ENUMS   
typedef enum {                          //Channel state
        TFTPCHANNEL_STATE_INVALID  = 0  //  Invalid - should never occur    
       ,TFTPCHANNEL_STATE_NEW           //  New object - no transfer attempted
       ,TFTPCHANNEL_STATE_GET_ACTIVE    //  GET in progress (active)
       ,TFTPCHANNEL_STATE_GET_DONE_OK   //  GET ended with success
       ,TFTPCHANNEL_STATE_GET_DONE_ERR  //  GET ended with error
       ,TFTPCHANNEL_STATE_GET_DONE_CAN  //  GET ended with cancel
       ,TFTPCHANNEL_STATE_PUT_ACTIVE    //  PUT in progress (active)
       ,TFTPCHANNEL_STATE_PUT_DONE_OK   //  PUT ended with success
       ,TFTPCHANNEL_STATE_PUT_DONE_ERR  //  PUT ended with error
       ,TFTPCHANNEL_STATE_PUT_DONE_CAN  //  PUT ended with cancel
    }   TFTPCHANNEL_STATE;

typedef 
enum tabTftpErrCod
    { ITC_TFTPERRCOD_S_NOT_DEFINED  = 0,
   ITC_TFTPERRCOD_S_FILE_NOTFOUND   = 1,
   ITC_TFTPERRCOD_S_ACCESS_ERROR = 2,
   ITC_TFTPERRCOD_S_DISK_FULL = 3,
   ITC_TFTPERRCOD_S_ILLEGAL_OP   = 4,
   ITC_TFTPERRCOD_S_UNK_TRANS_ID = 5,
   ITC_TFTPERRCOD_S_FILE_EXISTS  = 6,
   ITC_TFTPERRCOD_S_NO_SUCH_USER = 7,
   ITC_TFTPERRCOD_S_OPT_EXT_ERR  = 8,
   ITC_TFTPERRCOD_C_NOT_DEFINED  = 80,
   ITC_TFTPERRCOD_C_FILE_NOTFOUND   = 81,
   ITC_TFTPERRCOD_C_ILLEGAL_OP   = 84,
   ITC_TFTPERRCOD_C_WRITEFAIL = 85,
   ITC_TFTPERRCOD_C_RESENDLIMIT  = 97,
   ITC_TFTPERRCOD_C_CANCELED  = 98
    } ITC_TFTPERRCOD;


typedef enum {                          //Error source
        TFTPERR_SOURCE_NONE        = 0  //  No error set
       ,TFTPERR_SOURCE_SERVER           //  Error set by TFTP server ERROR packet
       ,TFTPERR_SOURCE_CLIENT           //  Error set by this Client
    }   TFTPERR_SOURCE;

typedef enum {                          //Cast all WSA errors into enum to make step through debugging a bit easier
        WSAERR_0                   = 0
       ,WSAERR_INTR                = WSAEINTR           
       ,WSAERR_BADF                = WSAEBADF           
       ,WSAERR_ACCES               = WSAEACCES          
       ,WSAERR_FAULT               = WSAEFAULT          
       ,WSAERR_INVAL               = WSAEINVAL          
       ,WSAERR_MFILE               = WSAEMFILE              //No more socket descriptors are available
       ,WSAERR_WOULDBLOCK          = WSAEWOULDBLOCK     
       ,WSAERR_INPROGRESS          = WSAEINPROGRESS     
       ,WSAERR_ALREADY             = WSAEALREADY        
       ,WSAERR_NOTSOCK             = WSAENOTSOCK        
       ,WSAERR_DESTADDRREQ         = WSAEDESTADDRREQ    
       ,WSAERR_MSGSIZE             = WSAEMSGSIZE        
       ,WSAERR_PROTOTYPE           = WSAEPROTOTYPE      
       ,WSAERR_NOPROTOOPT          = WSAENOPROTOOPT     
       ,WSAERR_PROTONOSUPPORT      = WSAEPROTONOSUPPORT 
       ,WSAERR_SOCKTNOSUPPORT      = WSAESOCKTNOSUPPORT 
       ,WSAERR_OPNOTSUPP           = WSAEOPNOTSUPP      
       ,WSAERR_PFNOSUPPORT         = WSAEPFNOSUPPORT    
       ,WSAERR_AFNOSUPPORT         = WSAEAFNOSUPPORT    
       ,WSAERR_ADDRINUSE           = WSAEADDRINUSE      
       ,WSAERR_ADDRNOTAVAIL        = WSAEADDRNOTAVAIL   
       ,WSAERR_NETDOWN             = WSAENETDOWN        
       ,WSAERR_NETUNREACH          = WSAENETUNREACH     
       ,WSAERR_NETRESET            = WSAENETRESET       
       ,WSAERR_CONNABORTED         = WSAECONNABORTED    
       ,WSAERR_CONNRESET           = WSAECONNRESET      
       ,WSAERR_NOBUFS              = WSAENOBUFS             //No buffer space is available
       ,WSAERR_ISCONN              = WSAEISCONN         
       ,WSAERR_NOTCONN             = WSAENOTCONN        
       ,WSAERR_SHUTDOWN            = WSAESHUTDOWN       
       ,WSAERR_TOOMANYREFS         = WSAETOOMANYREFS    
       ,WSAERR_TIMEDOUT            = WSAETIMEDOUT       
       ,WSAERR_CONNREFUSED         = WSAECONNREFUSED    
       ,WSAERR_LOOP                = WSAELOOP           
       ,WSAERR_NAMETOOLONG         = WSAENAMETOOLONG    
       ,WSAERR_HOSTDOWN            = WSAEHOSTDOWN       
       ,WSAERR_HOSTUNREACH         = WSAEHOSTUNREACH    
       ,WSAERR_NOTEMPTY            = WSAENOTEMPTY       
       ,WSAERR_PROCLIM             = WSAEPROCLIM        
       ,WSAERR_USERS               = WSAEUSERS          
       ,WSAERR_DQUOT               = WSAEDQUOT          
       ,WSAERR_STALE               = WSAESTALE          
       ,WSAERR_REMOTE              = WSAEREMOTE         
       ,WSAERR_DISCON              = WSAEDISCON         
    }   WSAERR;



////////////////////////////////////////////////////////////////
//   CLASSES
//
#ifdef __cplusplus

// forward declarations
class ITCTftpFileObject;
class TftpBuffer;

class ITCTftpClient  {

public:   
   ITCTftpClient();
    ~ITCTftpClient();
    ////////////////////////////////////////////////////////////////
    //
    // ITCTftpClient methods.  All methods are (and must be) implemented by this COM object.
    //
    ////////////////////////////////////////////////////////////////
    STDMETHODIMP Cancel(
        );

    STDMETHODIMP GetFile(   
        /* [in]  */ LPCTSTR         pszCltFileName,     //Filename on TFTP Client (required)
        /* [in]  */ LPCTSTR         pszSvrFileName      //Filename on TFTP Server (optional)
        );                                                                             

    STDMETHODIMP GetObject(
        /* [in]  */ ITCTftpFileObject *pTftpObject  //-> COM object implementing IITCTftpObject
        );                                                                             

    STDMETHODIMP GetProperty(
        /* [in]  */ ITC_TFTPPROP   eTftpProp,    //Property value desired
        /* [out] */ ULONG          *pulValue     //-> LONG to received value
        );  
                                                                                   
    STDMETHODIMP PutFile(   
        /* [in]  */ LPCTSTR         pszCltFileName,     //Filename on TFTP Client (required)
        /* [in]  */ LPCTSTR         pszSvrFileName      //Filename on TFTP Server (optional)
        );                                                                             

    STDMETHODIMP PutObject(
        /* [in]  */ ITCTftpFileObject *pTftpObject  //-> COM object implementing IITCTftpObject
        );                                                                             

    STDMETHODIMP SetProperty(
        /* [in]  */ ITC_TFTPPROP   eTftpProp,    //Property value being set
        /* [out] */ ULONG          ulValue       //new value
        );  
                                                                                   
    STDMETHODIMP SetServer( 
        /* [in]  */ LPCTSTR pszServerName,       // DNS name or IP address (xxx.xxx.xxx.xxx)
        /* [in]  */ WORD    wServerPort          // Port number (0 implies default)
        );                                                                             

    STDMETHODIMP Status(
        /* [out] */ ITC_TFTPSTATUS  *pStatus     //-> ITC_TFTPSTATUS structure
        );                                                                             
protected:
        //State of this object
    TFTPCHANNEL_STATE State()           { return m_eState; };

        //Allocate a TFTP buffer, freeing any prior buffer
    void            AllocBuffer(
                        WORD            wTftpSize   //Size of TFTP data area ( TFTP_BLOCK.wLen )
                       ,TFTP_BLOCK**    ppBlock     //->-> allocated block
                        );

        //Assemble WRQ or RRQ with Option Extensions
    bool            AssembleRequest(
                        const char*     pszAsciiName//-> ASCII filename (required)
                       ,ITC_TFTPMODE    eMode       //Transfer mode     (required)
                       ,const char*     pszUtf8Name //-> ASCII UTF8 filename      or null if none
                       ,WORD            wBlockSize  //Suggest blksize (RFC 2348)  or 0    if none
                       ,WORD            wTimeout    //Suggest timeout (RFC 2349)  or 0    if none
                       ,DWORD           wTsize      //Inform  filesize(RFC 2349)  or 0    if none
                       );

        //Initiate Server transfer - do WRQ or RRQ on server name
    bool            BeginTransfer( );

        //Free a TFTP_BLOCK buffer created by AllocBuffer
    void            FreeBuffer(
                        TFTP_BLOCK**    ppBlock     //->-> allocated block
                        );

        //Get registry settings and update internal settings as appropriate
    bool            GetRegistrySettings();

        //Test wheather Cancel has been set, in thread-safe fashion
    bool            IsCanceled();

        //Are we currently in a GET or PUT transfer
    bool            IsRunning();

        //Parse a TFTP Server response - primarily to get OACK BLKSIZE for RFC 2348
        //      and TIMEOUT for RFC 2349
        //Return - true if OACK, otherwise false
    bool            ParseOackResponse(
                        WORD*           pwTimeout   //-> WORD (set 0 if option not returned)
                       ,WORD*           pwBlkSize   //-> WORD (set 0 if option not returned)
                        );

        //Resolve the string server name to an IP address
    bool            ResolveIpAddr( );

        //Receive error packet from server (store for possible forward to owner)
    void            RecvErrorPacket(
                        TFTP_ERROR*     pErrPacket  //-> TFTP_ERROR packet from server
                        );

        //Reset state variables to begin a transfer.  Note that the ITCTftpClient is reuseable.
    void            ResetState( void );

        //Send an error packet to server, do not await response
    void            SendErrorPacket(
                        WORD            wErrCode    //Error code (TFTP_ERR_)
                       ,const char*     pszMsg );   //Error message (ASCII)

        //Send TFTP block, wait for acknowledge, handle retransmit
        //  On call m_pTBlock should contain block to send
        //Return true - m_pTBlock has block received from server, false - send failed
    bool            SendtoRecvfrom( 
                        ULONG       ulTimeout       //Timeout value to use
                       ,WORD        wResendLimit    //Limit value for #resends.  After this treat as error
                       ,WORD*       pwResent        //Update resend count here.  Can point to non-zero count on entry
                       ,WSAERR*     pErr            //Update final error, if return false
                        );

     
    #ifdef _DEBUG
        //Give text name for state
    LPCTSTR         StateName( 
                        TFTPCHANNEL_STATE eState ); //state
    #endif

        //Set the state and fire the state change event
    void            StateSet( 
                        TFTPCHANNEL_STATE eState ); //New state

        //Default Wait period - this needs to be revised    ----- REWORK NEEDED -----
    DWORD           WaitLockPeriod()    { return 5000; };

        

    CRITICAL_SECTION
                    m_critSection;      //Guard in multiple thread issues
    ITCTftpFileObject* m_pTObject;         //Object we are transferring
   WORD            m_wResendLimit;     //Resend limit to use
   ULONG           m_ulTimeout;        //Timeout limit to use
    WORD            m_wBlocksOk;        //# of successful TFTP blocks
    WORD            m_wBlocksFail;      //# of failed TFTP blocks
    DWORD           m_dwTicksLastIP;    //Tick-count of last send() or recv()
    TFTPCHANNEL_STATE
                    m_eState;           //State of this channel
    HANDLE          m_evtStateChange;   //Fired on each change of m_eState (manual reset)
    bool            m_bCancel;          //Has cancel been set
    bool            m_bXOptions;        //Have TFTP Extensions options been disabled ?

    HANDLE          m_hWDThread;        //Handle to WDThreadProc() thread
    bool            m_bEndWDThread;     //Signal WDThreadProc() to end
    HANDLE          m_hTranTrdHandle;   //Thread handle for thread doing active transfer
    DWORD           m_dwTransTrdID;     //ID of this thread

    sockaddr_in     m_sockaddr;         //TFTP Server IP address and UDP port
    SOCKET          m_socket;           //Socket to anchor protocol
    TftpBuffer*     m_pbufServerName;   //Server name from constructor
    WORD            m_wServerPort;      //Server port number

    TFTP_BLOCK*     m_pbufRecvFrom;     //RecvFrom buffer mapped to TFTP messages
    TFTP_BLOCK*     m_pbufSendTo;       //SendTo     "      "     "   "      "   
    WORD            m_wBSizeDesired;    //RFC 2348 Blocksize we desire
    WORD            m_wBSizeAgreed;     // "    "  blocksize agreed
    
    TFTPERR_SOURCE  m_eErrorSource;     //Is the error from Server or Client
    WORD            m_wErrorCode;       //Final TFTP error code - set by either server or client
    TftpBuffer*     m_pbufErrorMsg;     //Final error message   - from either server or client     
                                        //Which properties did call set via SetProperty()
    bool            m_bSetProperty[ ITC_TFTPPROP_XOPTIONS ];     
};

#endif

#ifdef __cplusplus
extern "C" {
#endif
//------------------------------------------------------------------------------------------------------------------
// File Transfer functions
//------------------------------------------------------------------------------------------------------------------
typedef void TFTP_CALLBACK(HANDLE hTransferHandle);
typedef TFTP_CALLBACK* PTFTP_CALLBACK;

HRESULT ITCTransmitFile(LPCTSTR pszServer, LPCTSTR pszClientFileName, 
                                LPCTSTR pszServerFileName);
HRESULT ITCReceiveFile(LPCTSTR pszServer, LPCTSTR pszClientFileName, 
                                LPCTSTR pszServerFileName);

#ifdef __cplusplus

HRESULT ITCTransmitFileEx(LPCTSTR pszServer, LPCTSTR pszClientFileName, 
                     LPCTSTR pszServerFileName, ITCTftpClient** ppITCTftpClient,
                     ITC_TFTPSTATUS*   pstatus);

HRESULT ITCReceiveFileEx(LPCTSTR pszServer, LPCTSTR pszClientFileName, 
                     LPCTSTR pszServerFileName, ITCTftpClient** ppITCTftpClient,
                     ITC_TFTPSTATUS*   pstatus);
#endif


// Thread safe C-interface
HRESULT ITCReceiveFileTFTP (LPCTSTR pszServer,LPCTSTR pszClientFileName, 
                        LPCTSTR pszServerFileName, HANDLE *transmitHandle,
                            PTFTP_CALLBACK pCallBack);

// Thread safe C-interface
HRESULT ITCTransmitFileTFTP (LPCTSTR pszServer, LPCTSTR pszClientFileName, 
                        LPCTSTR pszServerFileName, HANDLE *transmitHandle, 
                             PTFTP_CALLBACK pCallBack);

HRESULT ITCTransmitFileTFTPEx ( 
   LPCTSTR  pszServer,
   LPCTSTR pszClientFileName,
   LPCTSTR  pszServerFileName,
   HANDLE *transmitHandle,
    PTFTP_CALLBACK pCallBack,
    TFTP_EX_PARMS* exParms
    );

HRESULT ITCReceiveFileTFTPEx ( 
   LPCTSTR  pszServer,
   LPCTSTR pszClientFileName,
   LPCTSTR  pszServerFileName,
   HANDLE *transmitHandle,
    PTFTP_CALLBACK pCallBack,
    TFTP_EX_PARMS* exParms
    );

HRESULT ITCFileTransferStatusTFTP(ITC_TFTPSTATUS *pStatusObj, HANDLE transferHandle);
HRESULT ITCFileTransferCancelTFTP(HANDLE transferHandle);

//
// Functions related to the Audio Tone Generation API's (4/29/04)
// (A member of the DEVICE functional category).
//
BOOL     ITCIsAudioToneSupported( void );
HRESULT  ITCAudioPlayTone( 
   DWORD nPitch,     // IN: Freq in cycles per second (Hertz)
   DWORD nDuration,  // IN: Milliseconds.
   DWORD nVolume     // IN: A weighted range per specific device type.
   );

DWORD    ITCAudioToneGetCurrentVolume( void ) ;
//
DWORD ITC_GetTonePitchMin(void)        ;
DWORD ITC_GetTonePitchMax(void)        ;
DWORD ITC_GetTonePitchClick(void)      ;
DWORD ITC_GetTonePitchLow(void)        ;
DWORD ITC_GetTonePitchVeryLow(void)    ;
DWORD ITC_GetTonePitchHigh(void)       ;
DWORD ITC_GetTonePitchVeryHigh(void)   ;

DWORD ITC_GetToneDurationMin(void)     ;
DWORD ITC_GetToneDurationMax(void)     ;
DWORD ITC_GetToneDurationClick(void)   ;
DWORD ITC_GetToneDurationBeep(void)    ;

DWORD ITC_GetToneVolumeMin(void)       ;
DWORD ITC_GetToneVolumeMax(void)       ;
DWORD ITC_GetToneVolumeOff(void)       ;
DWORD ITC_GetToneVolumeLow(void)       ;
DWORD ITC_GetToneVolumeNormal(void)    ;
DWORD ITC_GetToneVolumeLoud(void)      ;
DWORD ITC_GetToneVolumeVeryLoud(void)  ;
DWORD ITC_ToneVolumeCurrent(void)      ;  // Returns 'ITC_TONE_VOLUME_CURRENT'
#define ITC_TONE_VOLUME_CURRENT     32    /* Use the current volume */

//------------------------------------------------------------------------------------------------------------------
// ITCLed functions
//------------------------------------------------------------------------------------------------------------------
typedef enum _LED_NO{
	LED_VM3_USER_GREEN = 12,
	LED_VM3_USER_RED = 13,
} LED_NO, *P_LED_NO;

//This function only supports on VM3 device
HRESULT	ITCLEDGetControl(int LedNo, BYTE *ActivePeriod, BYTE *BlinkRate);

//This function only supports on VM3 device
HRESULT ITCLEDSetControl(int LedNo, BYTE ActivePeriod, BYTE BlinkRate);

//

#ifdef __cplusplus
} // end of extern "C"
#endif

#endif
//
//  END OF FILE:  Itc50.h
//  Copyright (c) 1998-2015 Honeywell International Inc. All Rights Reserved.
//    
